<?php

namespace Drupal\workflows_diagram\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\workflows\WorkflowInterface;

/**
 * Returns responses for Workflows Diagram routes.
 */
class WorkflowsDiagramController extends ControllerBase {

  /**
   * Produce a Mermaid compatible string from a workflow.
   *
   * @param WorkflowInterface $workflow
   *   Workflow object.
   * @param array $classes
   *   Classes to apply per state or transition,
   *   keyed by 'transitions' and 'states'.
   * @param string $title
   *   Flowchart title.
   * @param array $disabled
   *   IDs of states or transitions to exclude from diagram,
   *   keyed by 'transitions' and 'states'.
   *
   * @return string
   *   Mermaid-compatible string.
   */
  public static function convertWorkflowToMermaid(WorkflowInterface $workflow, array $classes = NULL, string $title = NULL, array $disabled = NULL) {
    $workflowType = $workflow->getTypePlugin();
    $states = $workflowType->getStates();

    if (!$title) {
      $title = $workflow->label();
    }

    // We collect classes to add to the end:
    $mermaid_classes = '';

    // Start mermaid output string with graph type:
    $mermaid = 'graph TB;';

    // Initial state, put at start:
    $initialState = $workflowType->getInitialState();
    if ($initialState) {
      unset($states[$initialState->id()]);
      $states = [$initialState] + $states;
    }

    // States:
    foreach ($states as $state) {
      // Do not render state if disabled:
      if ($disabled && isset($disabled['states']) && in_array($state->id(), $disabled['states'])) {
        continue;
      }

      // Add state to mermaid diagram:
      $mermaid .= $state->id() . '[' . $state->label() . '];';

      // Add classes to mermaid diagram:
      if ($classes && isset($classes['states']) && isset($classes['states'][$state->id()])) {
        foreach ($classes['states'][$state->id()] as $class) {
          $mermaid_classes .= 'class ' . $state->id() . ' ' . $class . ';';
        }
      }
    }

    // Transitions:
    $transitions = $workflowType->getTransitions();
    foreach ($transitions as $transition) {
      // Do not render transition if disabled:
      if ($disabled && isset($disabled['transitions']) && in_array($transition->id(), $disabled['transitions'])) {
        continue;
      }

      foreach ($transition->from() as $from_transition) {
        $from = $from_transition->id();
        $to = $transition->to()->id();

        // Do not render transition if any of the involved states are disabled:
        if ($disabled && isset($disabled['states']) && count(array_intersect($disabled['states'], [$from, $to])) > 0) {
          continue;
        }

        // Create transition arrow in mermaid:
        $mermaid .= $from . '--->' . '|"' . $transition->label() . '"|' . $to . ';';

        // @todo not sure how to refer to transitions with this, may not be possible:
        // if ($classes && isset($classes['transitions']) && isset($classes['transitions'][$transition->id()])) {
        //   foreach ($classes['transitions'][$transition->id()] as $class) {
        //     $mermaid_classes .= 'class ' . $transition->id() . ' ' . trim($class) . ';';
        //   }
        // }
      }
    }

    // To avoid hitting "end" issue with a state or transition named 'end':
    $mermaid = str_replace(';end[', ';End[', $mermaid);
    $mermaid = str_replace(';end-', ';End-', $mermaid);
    $mermaid = str_replace('|end|', '|End|', $mermaid);
    $mermaid = str_replace('|end;', '|End;', $mermaid);

    // Add classes to end of mermaid string:
    $mermaid .= $mermaid_classes;

    return $mermaid;
  }
}
